@@ -1,4 +1,3 @@
-#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 #
 # Copyright 2002-2011 Zuza Software Foundation
@@ -21,13 +20,9 @@
 """class that handles all header functions for a header in a po file"""
 
 import re
+import six
 import time
-
-try:
-    from collections import OrderedDict
-except ImportError:
-    # Python <= 2.6 fallback
-    from translate.misc.dictutils import ordereddict as OrderedDict
+from collections import OrderedDict
 
 from translate import __version__
 from translate.misc.dictutils import cidict
@@ -46,7 +41,8 @@ default_header = {
 
 def parseheaderstring(input):
     """Parses an input string with the definition of a PO header and returns
-    the interpreted values as a dictionary."""
+    the interpreted values as a dictionary.
+    """
     headervalues = OrderedDict()
     for line in input.split("\n"):
         if not line or ":" not in line:
@@ -61,7 +57,8 @@ def parseheaderstring(input):
 def tzstring():
     """Returns the timezone as a string in the format [+-]0000, eg +0200.
 
-    :rtype: str"""
+    :rtype: str
+    """
     if time.daylight:
         tzoffset = time.altzone
     else:
@@ -75,8 +72,8 @@ def tzstring():
 
 
 def update(existing, add=False, **kwargs):
-    """Update an existing header dictionary with the values in kwargs, adding new values
-    only if add is true.
+    """Update an existing header dictionary with the values in kwargs, adding
+    new values only if add is true.
 
     :return: Updated dictionary of header entries
     :rtype: dict of strings
@@ -98,8 +95,8 @@ def update(existing, add=False, **kwargs):
             removed.append(key)
         elif add and key in fixedargs:
             headerargs[key] = fixedargs.pop(key)
-    for key, value in existing.iteritems():
-        if not key in removed:
+    for key, value in six.iteritems(existing):
+        if key not in removed:
             headerargs[key] = value
     if add:
         for key in fixedargs:
@@ -109,8 +106,9 @@ def update(existing, add=False, **kwargs):
 
 class poheader(object):
     """This class implements functionality for manipulation of po file headers.
-    This class is a mix-in class and useless on its own. It must be used from all
-    classes which represent a po file"""
+    This class is a mix-in class and useless on its own. It must be used from
+    all classes which represent a po file
+    """
 
     x_generator = "Translate Toolkit %s" % __version__.sver
 
@@ -126,7 +124,9 @@ class poheader(object):
         "Content-Type",
         "Content-Transfer-Encoding",
         "Plural-Forms",
+        "X-Accelerator-Marker",
         "X-Generator",
+        "X-Merge-On",
     ]
 
     def init_headers(self, charset='UTF-8', encoding='8bit', **kwargs):
@@ -137,17 +137,17 @@ class poheader(object):
         return self.header()
 
     def makeheaderdict(self,
-            charset="CHARSET",
-            encoding="ENCODING",
-            project_id_version=None,
-            pot_creation_date=None,
-            po_revision_date=None,
-            last_translator=None,
-            language_team=None,
-            mime_version=None,
-            plural_forms=None,
-            report_msgid_bugs_to=None,
-            **kwargs):
+                       charset="CHARSET",
+                       encoding="ENCODING",
+                       project_id_version=None,
+                       pot_creation_date=None,
+                       po_revision_date=None,
+                       last_translator=None,
+                       language_team=None,
+                       mime_version=None,
+                       plural_forms=None,
+                       report_msgid_bugs_to=None,
+                       **kwargs):
         """Create a header dictionary with useful defaults.
 
         pot_creation_date can be None (current date) or a value (datetime or string)
@@ -159,15 +159,15 @@ class poheader(object):
         """
         if project_id_version is None:
             project_id_version = "PACKAGE VERSION"
-        if pot_creation_date is None or pot_creation_date == True:
+        if pot_creation_date is None or pot_creation_date is True:
             pot_creation_date = time.strftime("%Y-%m-%d %H:%M") + tzstring()
         if isinstance(pot_creation_date, time.struct_time):
             pot_creation_date = time.strftime("%Y-%m-%d %H:%M", pot_creation_date) + tzstring()
         if po_revision_date is None:
             po_revision_date = "YEAR-MO-DA HO:MI+ZONE"
-        elif po_revision_date == False:
+        elif po_revision_date is False:
             po_revision_date = pot_creation_date
-        elif po_revision_date == True:
+        elif po_revision_date is True:
             po_revision_date = time.strftime("%Y-%m-%d %H:%M") + tzstring()
         if isinstance(po_revision_date, time.struct_time):
             po_revision_date = time.strftime("%Y-%m-%d %H:%M", po_revision_date) + tzstring()
@@ -197,9 +197,10 @@ class poheader(object):
         return update(defaultargs, add=True, **kwargs)
 
     def header(self):
-        """Returns the header element, or None. Only the first element is allowed
-        to be a header. Note that this could still return an empty header element,
-        if present."""
+        """Returns the header element, or None. Only the first element is
+        allowed to be a header. Note that this could still return an empty
+        header element, if present.
+        """
         if len(self.units) == 0:
             return None
         candidate = self.units[0]
@@ -210,7 +211,8 @@ class poheader(object):
 
     def parseheader(self):
         """Parses the PO header and returns the interpreted values as a
-        dictionary."""
+        dictionary.
+        """
         header = self.header()
         if not header:
             return {}
@@ -219,7 +221,8 @@ class poheader(object):
     def updateheader(self, add=False, **kwargs):
         """Updates the fields in the PO style header.
 
-        This will create a header if add == True."""
+        This will create a header if add == True.
+        """
         header = self.header()
         if not header:
             if add:
@@ -228,9 +231,9 @@ class poheader(object):
         else:
             headeritems = update(self.parseheader(), add, **kwargs)
             keys = headeritems.keys()
-            if not "Content-Type" in keys or "charset=CHARSET" in headeritems["Content-Type"]:
+            if "Content-Type" not in keys or "charset=CHARSET" in headeritems["Content-Type"]:
                 headeritems["Content-Type"] = "text/plain; charset=UTF-8"
-            if not "Content-Transfer-Encoding" in keys or "ENCODING" in headeritems["Content-Transfer-Encoding"]:
+            if "Content-Transfer-Encoding" not in keys or "ENCODING" in headeritems["Content-Transfer-Encoding"]:
                 headeritems["Content-Transfer-Encoding"] = "8bit"
             headerString = ""
             for key, value in headeritems.items():
@@ -268,7 +271,7 @@ class poheader(object):
 
     def updateheaderplural(self, nplurals, plural):
         """Update the Plural-Form PO header."""
-        if isinstance(nplurals, basestring):
+        if isinstance(nplurals, six.string_types):
             nplurals = int(nplurals)
         self.updateheader(add=True, Plural_Forms="nplurals=%d; plural=%s;" % (nplurals, plural))
 
@@ -308,7 +311,7 @@ class poheader(object):
         :param lang: the new target language code
         :type lang: str
         """
-        if isinstance(lang, basestring) and len(lang) > 1:
+        if isinstance(lang, six.string_types) and len(lang) > 1:
             self.updateheader(add=True, Language=lang, X_Poedit_Language=None, X_Poedit_Country=None)
 
     def getprojectstyle(self):
@@ -350,9 +353,7 @@ class poheader(object):
         :param project_style: the new project
         :type project_style: str
         """
-        from translate.filters.checks import projectcheckers
-        if project_style in projectcheckers:
-            self.updateheader(add=True, X_Project_Style=project_style)
+        self.updateheader(add=True, X_Project_Style=project_style)
 
     def mergeheaders(self, otherstore):
         """Merges another header with this header.
@@ -427,7 +428,8 @@ class poheader(object):
     def makeheader(self, **kwargs):
         """Create a header for the given filename.
 
-        Check .makeheaderdict() for information on parameters."""
+        Check .makeheaderdict() for information on parameters.
+        """
         headerpo = self.UnitClass("", encoding=self._encoding)
         headerpo.markfuzzy()
         headeritems = self.makeheaderdict(**kwargs)
