@@ -1,6 +1,7 @@
-#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
+from __future__ import print_function, unicode_literals
 
+import six
 from pytest import raises
 
 from translate.misc import wStringIO
@@ -33,25 +34,25 @@ class TestHelpers():
         # Newline handling
         assert pypo.quoteforpo("One\nTwo\n") == ['""', '"One\\n"', '"Two\\n"']
         # First line wrapping
-        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A ver") == \
-                             ['"A very long sentence. A very long sentence. A very long sentence. A ver"']
-        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very") == \
-                              ['""',
-                               '"A very long sentence. A very long sentence. A very long sentence. A very"']
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A ver") == [
+            '"A very long sentence. A very long sentence. A very long sentence. A ver"']
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very") == [
+            '""',
+            '"A very long sentence. A very long sentence. A very long sentence. A very"']
         # Long line with a newline
-        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very lon\n") == \
-                             ['""', '"A very long sentence. A very long sentence. A very long sentence. A very "', '"lon\\n"']
-        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very 123\n") == \
-                             ['""', '"A very long sentence. A very long sentence. A very long sentence. A very "', '"123\\n"']
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very lon\n") == [
+            '""', '"A very long sentence. A very long sentence. A very long sentence. A very "', '"lon\\n"']
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very 123\n") == [
+            '""', '"A very long sentence. A very long sentence. A very long sentence. A very "', '"123\\n"']
         # Special 77 char failure.
-        assert pypo.quoteforpo("Ukuba uyayiqonda into eyenzekayo, \nungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. \n<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu \nobhucabhuca ukudibanisa kwakho.</b>") == \
-                             ['""',
-                              '"Ukuba uyayiqonda into eyenzekayo, \\n"',
-                              '"ungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. "',
-                              '"\\n"',
-                              '"<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu "',
-                              '"\\n"',
-                              '"obhucabhuca ukudibanisa kwakho.</b>"']
+        assert pypo.quoteforpo("Ukuba uyayiqonda into eyenzekayo, \nungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. \n<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu \nobhucabhuca ukudibanisa kwakho.</b>") == [
+            '""',
+            '"Ukuba uyayiqonda into eyenzekayo, \\n"',
+            '"ungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. "',
+            '"\\n"',
+            '"<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu "',
+            '"\\n"',
+            '"obhucabhuca ukudibanisa kwakho.</b>"']
 
     def test_quoteforpo_escaped_quotes(self):
         """Ensure that we don't break \" in two when wrapping
@@ -119,7 +120,8 @@ class TestPYPOUnit(test_po.TestPOUnit):
         assert unit.getnotes("developer") == "Verb"
         assert unit.getnotes("translator") == "Which meaning of file?\nThank you"
         assert unit.getnotes() == "Which meaning of file?\nThank you\nVerb"
-        assert raises(ValueError, unit.getnotes, "devteam")
+        with raises(ValueError):
+            unit.getnotes("devteam")
 
     def test_notes_withcomments(self):
         """tests that when we add notes that look like comments that we treat them properly"""
@@ -135,13 +137,11 @@ class TestPYPOUnit(test_po.TestPOUnit):
         str_max = "123456789 123456789 123456789 123456789 123456789 123456789 123456789 1"
         unit = self.UnitClass(str_max)
         expected = 'msgid "%s"\nmsgstr ""\n' % str_max
-        print(expected, str(unit))
         assert str(unit) == expected
         # at this length we wrap
         str_wrap = str_max + '2'
         unit = self.UnitClass(str_wrap)
         expected = 'msgid ""\n"%s"\nmsgstr ""\n' % str_wrap
-        print(expected, str(unit))
         assert str(unit) == expected
 
     def test_wrap_on_newlines(self):
@@ -150,7 +150,6 @@ class TestPYPOUnit(test_po.TestPOUnit):
         postring = ('"123456789\\n"\n' * 3)[:-1]
         unit = self.UnitClass(string)
         expected = 'msgid ""\n%s\nmsgstr ""\n' % postring
-        print(expected, str(unit))
         assert str(unit) == expected
 
         # Now check for long newlines segments
@@ -165,7 +164,6 @@ class TestPYPOUnit(test_po.TestPOUnit):
 msgstr ""
 '''
         unit = self.UnitClass(longstring)
-        print(expected, str(unit))
         assert str(unit) == expected
 
     def test_wrap_on_max_line_length(self):
@@ -173,10 +171,17 @@ msgstr ""
         string = "1 3 5 7 N " * 11
         expected = 'msgid ""\n%s\nmsgstr ""\n' % '"1 3 5 7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 "\n"7 N 1 3 5 7 N 1 3 5 7 N 1 3 5 7 N "'
         unit = self.UnitClass(string)
-        print("Expected:")
-        print(expected)
-        print("Actual:")
-        print(str(unit))
+        assert str(unit) == expected
+
+    def test_wrap_on_slash(self):
+        """test that we wrap on /"""
+        string = "1/3/5/7/N/" * 11
+        expected = '''msgid ""
+"1/3/5/7/N/1/3/5/7/N/1/3/5/7/N/1/3/5/7/N/1/3/5/7/N/1/3/5/7/N/1/3/5/7/N/1/3/5/"
+"7/N/1/3/5/7/N/1/3/5/7/N/1/3/5/7/N/"
+msgstr ""
+'''
+        unit = self.UnitClass(string)
         assert str(unit) == expected
 
     def test_spacing_max_line(self):
@@ -189,10 +194,6 @@ msgstr ""
 msgstr ""
 '''
         unit = self.UnitClass(idstring)
-        print("Expected:")
-        print(expected)
-        print("Actual:")
-        print(str(unit))
         assert str(unit) == expected
 
 
@@ -206,7 +207,7 @@ class TestPYPOFile(test_po.TestPOFile):
         thepo = pofile.units[0]
         thepo.msgidcomments.append('"_: first comment\\n"')
         thepo.msgidcomments.append('"_: second comment\\n"')
-        regenposource = str(pofile)
+        regenposource = bytes(pofile).decode('utf-8')
         assert regenposource.count("_:") == 1
 
     def test_merge_duplicates_msgctxt(self):
@@ -227,8 +228,6 @@ class TestPYPOFile(test_po.TestPOFile):
         assert len(pofile.units) == 2
         pofile.removeduplicates("merge")
         assert len(pofile.units) == 2
-        print(pofile.units[0].msgidcomments)
-        print(pofile.units[1].msgidcomments)
         assert pypo.unquotefrompo(pofile.units[0].msgidcomments) == "_: source1\n"
         assert pypo.unquotefrompo(pofile.units[1].msgidcomments) == "_: source2\n"
 
@@ -237,19 +236,19 @@ class TestPYPOFile(test_po.TestPOFile):
         posource = u'''#: nb\nmsgid "Norwegian Bokm\xe5l"\nmsgstr ""\n'''
         pofile = self.StoreClass(wStringIO.StringIO(posource.encode("UTF-8")), encoding="UTF-8")
         assert len(pofile.units) == 1
-        print(str(pofile))
+        print(bytes(pofile))
         thepo = pofile.units[0]
-        assert str(thepo) == posource.encode("UTF-8")
-        # extra test: what if we set the msgid to a unicode? this happens in prop2po etc
-        thepo.source = u"Norwegian Bokm\xe5l"
-        assert str(thepo) == posource.encode("UTF-8")
+        # On Python 2, str() returns bytestrings while on Python 3, str() returns unicode
+        assert str(thepo) == posource if six.PY3 else posource.encode("UTF-8")
+        assert six.text_type(thepo) == posource
         # Now if we set the msgstr to Unicode
         # this is an escaped half character (1/2)
-        halfstr = "\xbd ...".decode("latin-1")
+        halfstr = b"\xbd ...".decode("latin-1")
         thepo.target = halfstr
-        assert halfstr in str(thepo).decode("UTF-8")
+        assert halfstr in six.text_type(thepo)
         thepo.target = halfstr.encode("UTF-8")
-        assert halfstr.encode("UTF-8") in str(thepo)
+        if six.PY2:
+            assert halfstr.encode("UTF-8") in str(thepo)
 
     def test_posections(self):
         """checks the content of all the expected sections of a PO message"""
@@ -257,7 +256,7 @@ class TestPYPOFile(test_po.TestPOFile):
         pofile = self.poparse(posource)
         print(pofile)
         assert len(pofile.units) == 1
-        assert str(pofile) == posource
+        assert bytes(pofile).decode('utf-8') == posource
         assert pofile.units[0].othercomments == ["# other comment\n"]
         assert pofile.units[0].automaticcomments == ["#. automatic comment\n"]
         assert pofile.units[0].sourcecomments == ["#: source comment\n"]
@@ -267,7 +266,7 @@ class TestPYPOFile(test_po.TestPOFile):
         """tests behaviour of unassociated comments."""
         oldsource = '# old lonesome comment\n\nmsgid "one"\nmsgstr "een"\n'
         oldfile = self.poparse(oldsource)
-        print(str(oldfile))
+        print(bytes(oldfile))
         assert len(oldfile.units) == 1
 
     def test_prevmsgid_parse(self):
@@ -321,4 +320,70 @@ msgstr[1] "toetse"
         assert pofile.units[4].prev_msgctxt == [u'"context 2"']
         assert pofile.units[4].prev_source == multistring([u"tast", u"tasts"])
 
-        assert str(pofile) == posource
+        assert bytes(pofile).decode('utf-8') == posource
+
+    def test_wrap(self):
+        hello = ' '.join(['Hello'] * 100)
+        posource = 'msgid "{0}"\nmsgstr "{0}"\n'.format(hello).encode('utf-8')
+
+        # Instance with no line wraps
+        store = self.StoreClass(width=-1)
+        store.parse(posource)
+        assert bytes(store) == posource
+        store.units[0].settarget(hello)
+        assert bytes(store) == posource
+
+        # Instance with long line wraps
+        store = self.StoreClass(width=1000)
+        store.parse(posource)
+        assert bytes(store) == posource
+        store.units[0].settarget(hello)
+        assert bytes(store) == posource
+
+        # Instance with default wraps (77)
+        store = self.StoreClass()
+        store.parse(posource)
+        assert bytes(store) == posource
+        store.units[0].settarget('Hello ' * 100)
+        # Should contain additional newlines now
+        assert bytes(store) != posource
+
+    def test_wrap_newlines(self):
+        hello = '\n '.join(['Hello'] * 100)
+        posource = 'msgid "{0}"\nmsgstr "{0}"\n'.format(hello.replace('\n', '\\n')).encode('utf-8')
+
+        # Instance with no line wraps
+        store = self.StoreClass(width=-1)
+        store.parse(posource)
+        assert bytes(store) == posource
+        store.units[0].settarget(hello)
+        assert bytes(store) == posource
+
+        # Instance with default wraps (77)
+        store = self.StoreClass()
+        store.parse(posource)
+        assert bytes(store) == posource
+        store.units[0].settarget('Hello ' * 100)
+        # Should contain additional newlines now
+        assert bytes(store) != posource
+
+    def test_unix_newlines(self):
+        """checks that unix newlines are properly parsed"""
+        posource = b'\nmsgid "test me"\nmsgstr ""\n'
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 1
+        assert pofile.units[0].source == 'test me'
+
+    def test_dos_newlines(self):
+        """checks that dos newlines are properly parsed"""
+        posource = b'\r\nmsgid "test me"\r\nmsgstr ""\r\n'
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 1
+        assert pofile.units[0].source == 'test me'
+
+    def test_mac_newlines(self):
+        """checks that mac newlines are properly parsed"""
+        posource = b'\rmsgid "test me"\rmsgstr ""\r'
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 1
+        assert pofile.units[0].source == 'test me'
